# $Date: 1996/10/29 13:01:08 $  $Author: andi $  $Revision: 1.14 $ #

# pref.mu -- Andreas Sorgatz (andi@uni-paderborn.de), 20.11.1995 #

#++
   pref.mu

   pref -- Set or query MuPAD preferences

   pref( )
   pref( prefs )

   prefs  - a list, sequence, set or table of 'arg'
   arg    - is either of the form ident=value or simply an ident

   For every 'arg' of the form 'ident', the function returns the current 
   value of the preference 'ident'. For every equation 'ident=value',the
   function sets the preference 'ident' to the new value 'value'. If the 
   pereference 'ident' is undefined or the 'value' is out of range, this
   function returns FAIL.
  
   If no argument is given,  a table of all known preferences with their
   current values is returned.
   If the argument NIL is given,all preferences are set the their default
   values.

   The list of currently supported preferences is: 

   NAME      VALUES          DESCRIPTION
   ---------------------------------------------------------------------
   Report    0..9            0 - (default) Turn of periodical reports
                             1..9 - Print memory and time report during
                             evaluation: 1=seldom .. 9=often
                                      
   Prompt    TRUE/FALSE      TRUE - (default) Print the user prompt
                             FALSE - Do not print the user prompt
                             This preference is only available for
                             the tty version of MuPAD for UNIX.
                              
   PromptString    CHAR      Character string of the KErnel prompt

   Echo      TRUE/FALSE      TRUE - (default) Echo the user input
                             FALSE - Do not echo the user input
                             This preference is only available for
                             the tty version of MuPAD for UNIX.
                              
   Maxmem    0..x            0 - (default) No memory limit
                             If the MuPAD memory management allocates more
                             than x kbytes memory, the current calculation
                             is stopped as fast as possible. This feature
                             is inactiv during numerical calculations and
                             while reading files.
   
   Maxtime   0..x            0 - (default) keine Beschraenkung
                             If a MuPAD evaluation needs more than x Seconds,
                             it is stopped as fast as possible. This feature
                             is inactiv during numerical calculations and
                             while reading files.

   FloatFormat "g"/"f"/"e"   The output of DOM_FLOATs will be presented in
                             "f"  -- always decimal
                             "e"  -- always exponential
                             "g"  -- mix of the above, switch at 2^(-32)
			     
   Output    Expression      The output of data will be the result of the
                             evaluation of Expression(data)
			     
   PostOutput  Expression    Each output is followed by the result of the
                             evaluation of Expression(data)
   ---------------------------------------------------------------------

   Example:
   
        >> pref(Report=1):
        >> int(sin(x)*cos(x),x);
        [blocks=21075, bytes=1210404, seconds=0]
        [blocks=23659, bytes=1337116, seconds=1]
	
                         cos(2 x)
                       - --------
                             4
                             
	>> pref(Report, Prompt, Echo);
                             
	               0, TRUE, TRUE                   
++#

pref:= proc()
    local KPrefs, KDefs, LPrefs, LDefs;
begin
    ### Without any argument return the complete table ###
    ### of preferences and their current values.       ###

    ### The values in KPrefs are set by the kernel. ###
    KPrefs:= 
      [Report,     Maxtime,   Maxmem, PromptString,     Prompt, 
         Echo, FloatFormat, CallBack,       Output, PostOutput];
    KDefs := 
      [     0,           0,        0,        ">> ",       TRUE, 
         TRUE,         "g",    FALSE,          NIL,        NIL];

    ### The values in LPrefs are set by the library. ###
    LPrefs:= [ VerboseRead ];
    LDefs := [           0 ];

    if( args(0) = 0 ) then   ### print all preferences ###
      return( table( 
        op(zip(KPrefs, map(KPrefs, _pref), _equal)),
        op(stdlib::PREFTAB),
        null() ) );
    end_if;

    ### Call _pref according to the type of arguments. ###
    
    case type( args() )
      of DOM_NIL do ### Reset all internal Preferences ###
         pref(op(zip(KPrefs, KDefs, _equal)));
         pref(op(zip(LPrefs, LDefs, _equal)));
         break

      of DOM_SET    do
      of DOM_LIST   do
      of "_exprseq" do          ### Multiple Arguments ###
         return( map(args(), pref) );

      of DOM_TABLE  do
         if nops(args()) = 1 then
           return(pref(op(args())))
         else
           return(map(op(args()), pref))
         end_if;
         break;

      of DOM_IDENT  do                 ### get a value ###
        if contains(stdlib::PREFTAB, args()) then
          stdlib::PREFTAB[args()];
        elif contains( KPrefs, args())>0 then
          _pref(args())
        else
          FAIL
        end_if;
        break;

      of "_equal" do                   ### set a value ###
        if   domtype(op(args(),2)) = DOM_NIL and contains(KPrefs,op(args(),1))>0 then
           _pref( op(args(),1) = op(KDefs,contains(KPrefs,op(args(),1))) );
        elif domtype(op(args(),2)) = DOM_NIL and contains(LPrefs,op(args(),1))>0 then
           pref( op(args(),1) = op(LDefs,contains(LPrefs,op(args(),1))) );
        elif contains(KPrefs, op(args(), 1)) > 0 then 
           _pref( args() );
        else
           sysassign(stdlib::PREFTAB[op(args(), 1)], op(args(), 2));
        end_if;
        break;
      otherwise                     ### Any other case ###
        error("Illegal argument");
    end_case 
end_proc:

# end of file #
